'''
cbeams v{version}

I've seen things you people wouldn't believe. Attack ships on fire off the
shoulder of Orion. I watched c-beams glitter in the dark, near the Tannhäuser
Gate. All those moments will be lost, in time, like tears in rain. Time to die.

Usage:
    cbeams [ --help | --version | --overwrite ]

Options:
    -h --help       Show this text.
    -v --version    Print version number and exit.
    -o --overwrite  Overwrite terminal's visible screenful of content.

For details, see the README file.
'''

__version__ = '1.0.3'

__doc__ = __doc__.format(version=__version__)

